@@ -1,7 +1,7 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2005
+ * Copyright (c) 2006
  * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
  * 
  * This file is part of SIP.
@@ -35,7 +35,7 @@
 
 #define	DEFAULT_OFILE_EXT	".o"	/* Default object file extension. */
 
-#define	MAX_NR_ARGS		20	/* Max. nr. args. to a function. */
+#define	MAX_NR_ARGS		20	/* Max. nr. args. to a function or template. */
 
 
 /* For convenience. */
@@ -44,6 +44,17 @@
 #define	classFQCName(cd)	((cd) -> iff -> fqcname)
 
 
+/* Handle module flags. */
+
+#define	MOD_HAS_DELAYED_DTORS	0x0001	/* It has a class with a delayed dtor. */
+#define	MOD_IS_ORDERED		0x0002	/* Set if the module has been ordered. */
+
+#define	hasDelayedDtors(m)	((m) -> modflags & MOD_HAS_DELAYED_DTORS)
+#define	setHasDelayedDtors(m)	((m) -> modflags |= MOD_HAS_DELAYED_DTORS)
+#define	isOrdered(m)		((m) -> modflags & MOD_IS_ORDERED)
+#define	setIsOrdered(m)		((m) -> modflags |= MOD_IS_ORDERED)
+
+
 /* Handle section flags. */
 
 #define	SECT_IS_PUBLIC		0x0001	/* It is public. */
@@ -68,6 +79,8 @@
 #define	CLASS_IS_RENAMED	0x02000000	/* It has a different Python name. */
 #define	CLASS_IS_INCOMPLETE	0x04000000	/* The specification is incomplete. */
 #define	CLASS_CAN_CREATE	0x08000000	/* It has usable ctors. */
+#define	CLASS_IS_EXTERNAL	0x10000000	/* It is external. */
+#define	CLASS_IS_DELAYED_DTOR	0x20000000	/* The dtor is delayed. */
 
 #define	hasEnums(cd)		((cd) -> classflags & CLASS_HAS_ENUMS)
 #define	setHasEnums(cd)		((cd) -> classflags |= CLASS_HAS_ENUMS)
@@ -97,6 +110,10 @@
 #define	canCreate(cd)		((cd) -> classflags & CLASS_CAN_CREATE)
 #define	setCanCreate(cd)	((cd) -> classflags |= CLASS_CAN_CREATE)
 #define	resetCanCreate(cd)	((cd) -> classflags &= ~CLASS_CAN_CREATE)
+#define	isExternal(cd)		((cd) -> classflags & CLASS_IS_EXTERNAL)
+#define	setIsExternal(cd)	((cd) -> classflags |= CLASS_IS_EXTERNAL)
+#define	isDelayedDtor(cd)	((cd) -> classflags & CLASS_IS_DELAYED_DTOR)
+#define	setIsDelayedDtor(cd)	((cd) -> classflags |= CLASS_IS_DELAYED_DTOR)
 
 #define	isPublicDtor(cd)	((cd) -> classflags & SECT_IS_PUBLIC)
 #define	setIsPublicDtor(cd)	((cd) -> classflags |= SECT_IS_PUBLIC)
@@ -109,6 +126,8 @@
 /* Handle ctor flags. */
 
 #define	CTOR_RELEASE_GIL	0x00010000	/* The ctor releases the GIL. */
+#define	CTOR_EXPLICIT		0x00020000	/* The ctor is explicit. */
+#define	CTOR_CAST		0x00040000	/* The ctor is a cast. */
 
 #define	isPublicCtor(c)		((c) -> ctorflags & SECT_IS_PUBLIC)
 #define	setIsPublicCtor(c)	((c) -> ctorflags |= SECT_IS_PUBLIC)
@@ -119,6 +138,9 @@
 
 #define	isReleaseGILCtor(c)	((c) -> ctorflags & CTOR_RELEASE_GIL)
 #define	setIsReleaseGILCtor(c)	((c) -> ctorflags |= CTOR_RELEASE_GIL)
+#define	isExplicitCtor(c)	((c) -> ctorflags & CTOR_EXPLICIT)
+#define	setIsExplicitCtor(c)	((c) -> ctorflags |= CTOR_EXPLICIT)
+#define	isCastCtor(c)		((c) -> ctorflags & CTOR_CAST)
 
 
 /* Handle member flags. */
@@ -131,11 +153,18 @@
 
 /* Handle enum flags. */
 
-#define	ENUM_IS_RENAMED		0x0001
+#define	ENUM_WAS_PROT		0x00010000	/* It was defined as protected. */
+#define	ENUM_IS_RENAMED		0x00020000	/* It has been renamed. */
 
-#define	isRenamedEnum(e)	((e) -> enumflags & CLASS_IS_RENAMED)
-#define	setIsRenamedEnum(e)	((e) -> enumflags |= CLASS_IS_RENAMED)
-#define	isProtectedEnum(e)	((e) -> pcd != NULL)
+#define	isProtectedEnum(e)	((e) -> enumflags & SECT_IS_PROT)
+#define	setIsProtectedEnum(e)	((e) -> enumflags |= SECT_IS_PROT)
+#define	resetIsProtectedEnum(e)	((e) -> enumflags &= ~SECT_IS_PROT)
+
+#define	wasProtectedEnum(e)	((e) -> enumflags & ENUM_WAS_PROT)
+#define	setWasProtectedEnum(e)	((e) -> enumflags |= ENUM_WAS_PROT)
+#define	resetWasProtectedEnum(e)	((e) -> enumflags &= ~ENUM_WAS_PROT)
+#define	isRenamedEnum(e)	((e) -> enumflags & ENUM_IS_RENAMED)
+#define	setIsRenamedEnum(e)	((e) -> enumflags |= ENUM_IS_RENAMED)
 
 
 /* Handle hierarchy flags. */
@@ -161,6 +190,7 @@
 #define	OVER_IS_FACTORY		0x00400000	/* It is a factory method. */
 #define	OVER_XFERRED_BACK	0x00800000	/* Ownership is transferred back. */
 #define	OVER_RELEASE_GIL	0x01000000	/* The function releases the GIL. */
+#define	OVER_IS_VIRTUAL_REIMP	0x02000000	/* It is a re-implementation of a virtual. */
 
 #define	isPublic(o)		((o) -> overflags & SECT_IS_PUBLIC)
 #define	setIsPublic(o)		((o) -> overflags |= SECT_IS_PUBLIC)
@@ -192,6 +222,8 @@
 #define setIsResultTransferredBack(o)	((o) -> overflags |= OVER_XFERRED_BACK)
 #define	isReleaseGIL(o)		((o) -> overflags & OVER_RELEASE_GIL)
 #define	setIsReleaseGIL(o)	((o) -> overflags |= OVER_RELEASE_GIL)
+#define	isVirtualReimp(o)	((o) -> overflags & OVER_IS_VIRTUAL_REIMP)
+#define	setIsVirtualReimp(o)	((o) -> overflags |= OVER_IS_VIRTUAL_REIMP)
 
 
 /* Handle variable flags. */
@@ -246,6 +278,7 @@
 #define	setIsOutArg(a)		((a) -> argflags |= ARG_OUT)
 #define	isConstrained(a)	((a) -> argflags & ARG_CONSTRAINED)
 #define	setIsConstrained(a)	((a) -> argflags |= ARG_CONSTRAINED)
+#define	resetIsConstrained(a)	((a) -> argflags &= ~ARG_CONSTRAINED)
 
 
 /* Handle name flags. */
@@ -279,6 +312,8 @@ typedef enum {
 	str_slot,
 	unicode_slot,
 	int_slot,
+	long_slot,
+	float_slot,
 	len_slot,
 	contains_slot,
 	add_slot,
@@ -322,6 +357,7 @@ typedef enum {
 	pos_slot,
 	abs_slot,
 	repr_slot,
+	hash_slot,
 	no_slot
 } slotType;
 
@@ -330,7 +366,6 @@ typedef enum {
  * Argument types.  Always add new ones at the end because the numeric values
  * can appear in generated code.
  */
-
 typedef enum {
 	no_type,
 	defined_type,
@@ -367,7 +402,14 @@ typedef enum {
 	pycallable_type,
 	pyslice_type,
 	qobject_type,
-	function_type
+	function_type,
+	pytype_type,
+	ellipsis_type,
+	longlong_type,
+	ulonglong_type,
+	anyslot_type,
+	cbool_type,
+	sstring_type
 } argType;
 
 
@@ -459,8 +501,10 @@ typedef struct _nodeDef {
 /* A module definition. */
 
 typedef struct _moduleDef {
-	char			*name;		/* The module name. */
+	char			*fullname;	/* The full module name. */
+	char			*name;		/* The module base name. */
 	int			version;	/* The module version. */
+	int			modflags;	/* The module flags. */
 	int			modulenr;	/* The module number. */
 	char			*file;		/* The filename. */
 	qualDef			*qualifiers;	/* The list of qualifiers. */
@@ -520,15 +564,6 @@ typedef struct _exceptionDef {
 } exceptionDef;
 
 
-/* A function call. */
-
-typedef struct _fcallDef {
-	scopedNameDef		*name;		/* The name. */
-	int			nrArgs;		/* The number of arguments. */
-	struct _valueDef	*args[MAX_NR_ARGS];	/* The arguments. */
-} fcallDef;
-
-
 /* A value. */
 
 typedef struct _valueDef {
@@ -541,7 +576,7 @@ typedef struct _valueDef {
 		double		vreal;		/* Real value. */
 		char		*vstr;		/* String value. */
 		scopedNameDef	*vscp;		/* Scoped value. */
-		fcallDef	*fcd;		/* Function call. */
+		struct _fcallDef *fcd;		/* Function call. */
 	} u;
 	struct _valueDef	*next;		/* Next in the expression. */
 } valueDef;
@@ -551,6 +586,7 @@ typedef struct _valueDef {
 
 typedef struct {
 	argType			atype;		/* The type. */
+	char			*name;		/* The name. */
 	int			argflags;	/* The argument flags. */
 	int			nrderefs;	/* Nr. of dereferences. */
 	valueDef		*defval;	/* The default value. */
@@ -567,6 +603,22 @@ typedef struct {
 
 
 
+/* An entry in a linked argument list. */
+typedef struct _argList {
+	argDef 			arg;		/* The argument itself. */
+	struct _argList		*next;		/* Next in the list. */
+} argList;
+
+
+/* A function call. */
+
+typedef struct _fcallDef {
+	argDef			type;		/* The type. */
+	int			nrArgs;		/* The number of arguments. */
+	struct _valueDef	*args[MAX_NR_ARGS];	/* The arguments. */
+} fcallDef;
+
+
 /* An interface file definition. */
 
 typedef struct _ifaceFileDef {
@@ -583,6 +635,7 @@ typedef struct _ifaceFileDef {
 
 typedef struct _ifaceFileList {
 	ifaceFileDef		*iff;		/* The interface file itself. */
+	int			header;		/* If needed in the .h file. */
 	struct _ifaceFileList	*next;		/* Next in the list. */
 } ifaceFileList;
 
@@ -630,7 +683,8 @@ typedef struct _templateDef {
 typedef struct _virtHandlerDef {
 	int			virthandlernr;	/* The nr. of the virtual handler. */
 	int			vhflags;	/* The virtual handler flags. */
-	signatureDef		*sd;		/* The signature. */
+	signatureDef		*pysig;		/* The Python signature. */
+	signatureDef		*cppsig;	/* The C++ signature. */
 	struct _moduleDef	*module;	/* The defining module. */
 	codeBlock		*virtcode;	/* Virtual handler code. */
 	struct _virtHandlerDef	*next;		/* Next in the list. */
@@ -686,7 +740,7 @@ typedef struct _overDef {
 typedef struct _ctorDef {
 	int			ctorflags;	/* The ctor flags. */
 	signatureDef		pysig;		/* The Python signature. */
-	signatureDef		*cppsig;	/* The C++ signature. */
+	signatureDef		*cppsig;	/* The C++ signature, NULL if /NoDerived/. */
 	throwArgs		*exceptions;	/* The exceptions. */
 	codeBlock		*methodcode;	/* Method code. */
 	char			*prehook;	/* The pre-hook name. */
@@ -710,12 +764,14 @@ typedef struct _enumMemberDef {
 typedef struct _enumDef {
 	int			enumflags;	/* The enum flags. */
 	scopedNameDef		*fqcname;	/* The name (may be NULL). */
-	char			*pyname;	/* The Python name (may be NULL). */
+	nameDef			*pyname;	/* The Python name (may be NULL). */
 	int			enumnr;		/* The enum number. */
 	struct _classDef	*ecd;		/* The enclosing class. */
 	struct _classDef	*pcd;		/* The publishing class. */
 	moduleDef		*module;	/* The owning module. */
 	enumMemberDef		*members;	/* The list of members. */
+	struct _memberDef	*slots;		/* The list of slots. */
+	struct _overDef		*overs;		/* The list of slot overloads. */
 	struct _enumDef		*next;		/* Next in the list. */
 } enumDef;
 
@@ -774,9 +830,11 @@ typedef struct _classDef {
 	char			*pyname;	/* The Python name. */
 	ifaceFileDef		*iff;		/* The interface file. */
 	struct _classDef	*ecd;		/* The enclosing scope. */
+	struct _classDef	*real;		/* The real class if this is a proxy or extender. */
 	nodeDef			*node;		/* Position in class tree. */
 	classList		*supers;	/* The parent classes. */
 	mroDef			*mro;		/* The super-class hierarchy. */
+	templateDef		*td;		/* The instantiated template. */
 	ctorDef			*ctors;		/* The constructors. */
 	ctorDef			*defctor;	/* The default ctor. */
 	codeBlock		*dealloccode;	/* Handwritten dealloc code. */
@@ -784,6 +842,7 @@ typedef struct _classDef {
 	throwArgs		*dtorexceptions;	/* The dtor exceptions. */
 	memberDef		*members;	/* The member functions. */
 	overDef			*overs;		/* The overloads. */
+	argList			*casts;		/* The operator casts. */
 	virtOverDef		*vmembers;	/* The virtual members. */
 	visibleList		*visible;	/* The visible members. */
 	codeBlock		*cppcode;	/* Class C++ code. */
@@ -793,20 +852,46 @@ typedef struct _classDef {
 	codeBlock		*convtocode;	/* Convert to C++ code. */
 	codeBlock		*travcode;	/* Traverse code. */
 	codeBlock		*clearcode;	/* Clear code. */
+	codeBlock		*readbufcode;	/* Read buffer code. */
+	codeBlock		*writebufcode;	/* Write buffer code. */
+	codeBlock		*segcountcode;	/* Segment count code. */
+	codeBlock		*charbufcode;	/* Character buffer code. */
 	struct _classDef	*next;		/* Next in the list. */
 } classDef;
 
 
+/* A class template definition. */
+
+typedef struct _classTmplDef {
+	signatureDef		sig;		/* The template arguments. */
+	classDef		*cd;		/* The class itself. */
+	struct _classTmplDef	*next;		/* The next in the list. */
+} classTmplDef;
+
+
+/* A mapped type template definition. */
+
+typedef struct _mappedTypeTmplDef {
+	signatureDef		sig;		/* The template arguments. */
+	mappedTypeDef		*mt;		/* The mapped type itself. */
+	struct _mappedTypeTmplDef	*next;	/* The next in the list. */
+} mappedTypeTmplDef;
+
+
 /* The parse tree corresponding to the specification file. */
 
 typedef struct {
 	moduleDef		*module;	/* This module. */
 	moduleDef		*modules;	/* The list of modules. */
+	moduleListDef		*allimports;	/* The list of all imports. */
 	nameDef			*namecache;	/* The name cache. */
 	ifaceFileDef		*ifacefiles;	/* The list of interface files. */
 	classDef		*classes;	/* The list of classes. */
+	classTmplDef		*classtemplates;	/* The list of class templates. */
+	classDef		*proxies;	/* The list of proxy classes. */
 	exceptionDef		*exceptions;	/* The list of exceptions. */
 	mappedTypeDef		*mappedtypes;	/* The mapped types. */
+	mappedTypeTmplDef	*mappedtypetemplates;	/* The list of mapped type templates. */
 	int			qobjclass;	/* QObject class, -1 if none. */
 	enumDef			*enums;		/* List of enums. */
 	varDef			*vars;		/* List of variables. */
@@ -814,21 +899,24 @@ typedef struct {
 	overDef			*overs;		/* Global overloads. */
 	typedefDef		*typedefs;	/* List of typedefs. */
 	codeBlock		*copying;	/* Software license. */
+	codeBlock		*exphdrcode;	/* Exported header code. */
 	codeBlock		*hdrcode;	/* Header code. */
 	codeBlock		*cppcode;	/* Global C++ code. */
 	codeBlock		*docs;		/* Documentation. */
 	codeBlock		*preinitcode;	/* Pre-initialisation code. */
 	codeBlock		*postinitcode;	/* Post-initialisation code. */
+	codeBlock		*unitcode;	/* Compileation unit code. */
 	ifaceFileList		*used;		/* Interface files used. */
 	int			sigslots;	/* Set if signals or slots are used. */
 	int			genc;		/* Set if we are generating C code. */
+	struct _stringList	*options;	/* The list of options. */
 } sipSpec;
 
 
 /* A list of strings. */
 
 typedef struct _stringList {
-	char			*s;		/* The string. */
+	const char		*s;		/* The string. */
 	struct _stringList	*next;		/* The next in the list. */
 } stringList;
 
@@ -855,20 +943,33 @@ void fatalScopedName(scopedNameDef *);
 void setInputFile(FILE *,char *,parserContext *,int);
 void *sipMalloc(size_t);
 char *sipStrdup(char *);
-char *concat(char *,...);
+char *concat(const char *, ...);
 void append(char **,char *);
-void addToUsedList(ifaceFileList **,ifaceFileDef *);
+ifaceFileList *addToUsedList(ifaceFileList **, ifaceFileDef *);
 int excludedFeature(stringList *,qualDef *);
 int sameSignature(signatureDef *,signatureDef *,int);
+int sameTemplateSignature(signatureDef *sd1, signatureDef *sd2, int deep);
 int sameScopedName(scopedNameDef *,scopedNameDef *);
-int sameArgType(argDef *,argDef *,int);
-int sameBaseType(argDef *,argDef *,int);
+int sameBaseType(argDef *,argDef *);
 char *scopedNameTail(scopedNameDef *);
 scopedNameDef *text2scopePart(char *);
 scopedNameDef *copyScopedName(scopedNameDef *);
 void appendScopedName(scopedNameDef **,scopedNameDef *);
 void freeScopedName(scopedNameDef *);
 void appendToClassList(classList **,classDef *);
+void prOverloadName(FILE *fp, overDef *od);
+int isIntReturnSlot(memberDef *md);
+int isLongReturnSlot(memberDef *md);
+int isVoidReturnSlot(memberDef *md);
+int isNumberSlot(memberDef *md);
+int isRichCompareSlot(memberDef *md);
+mappedTypeDef *allocMappedType(argDef *type);
+void appendString(stringList **headp, const char *s);
+void appendTypeStrings(scopedNameDef *ename, signatureDef *patt, signatureDef *src, signatureDef *known, scopedNameDef **names, scopedNameDef **values);
+codeBlock *templateCode(sipSpec *pt, ifaceFileList **used, codeBlock *ocb, scopedNameDef *names, scopedNameDef *values);
+ifaceFileDef *findIfaceFile(sipSpec *pt, moduleDef *mod, scopedNameDef *fqname, ifaceFileType iftype, argDef *ad);
+int optNoEmitters(sipSpec *pt);
+int optRegisterTypes(sipSpec *pt);
 void yywarning(char *);
 
 
