@@ -1,7 +1,7 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2005
+ * Copyright (c) 2006
  * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
  * 
  * This file is part of SIP.
@@ -43,9 +43,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define	SIP_VERSION		0x040302
-#define	SIP_VERSION_STR		"4.3.2"
-#define	SIP_BUILD		"SIP-4_3-365"
+#define	SIP_VERSION		0x040405
+#define	SIP_VERSION_STR		"4.4.5"
 
 
 /*
@@ -58,6 +57,14 @@ extern "C" {
  *
  * History:
  *
+ * 3.2  Added sip_api_export_symbol() and sip_api_import_symbol().
+ *
+ * 3.1  Added sip_api_add_mapped_type_instance().
+ *
+ * 3.0  Moved the Qt support out of the sip module and into PyQt.  This is
+ *      such a dramatic change that there is no point in attempting to maintain
+ *      backwards compatibility.
+ *
  * 2.0	Added the td_flags field to the sipTypeDef structure.
  *	Added the first_child, sibling_next, sibling_prev and parent fields to
  *	the sipWrapper structure.
@@ -83,8 +90,8 @@ extern "C" {
  *
  * 0.0	Original version.
  */
-#define	SIP_API_MAJOR_NR	2
-#define	SIP_API_MINOR_NR	0
+#define	SIP_API_MAJOR_NR	3
+#define	SIP_API_MINOR_NR	2
 
 
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
@@ -126,6 +133,7 @@ typedef int sip_gilstate_t;
 typedef struct _sipWrapperType {
 	PyHeapTypeObject super;		/* The super-metatype. */
 	struct _sipTypeDef *type;	/* The additional type information. */
+	struct _sipInitExtenderDef *iextend;	/* The list of init extenders. */
 } sipWrapperType;
 
 
@@ -134,6 +142,7 @@ typedef struct _sipWrapperType {
  */
 typedef struct _sipWrapper {
 	PyObject_HEAD
+	PyObject *user;			/* For the user to use. */
 	union {
 		void *cppPtr;		/* C/C++ object pointer. */
 		void *(*afPtr)();	/* Access function. */
@@ -152,17 +161,30 @@ typedef struct _sipWrapper {
 /*
  * Some convenient function pointers.
  */
-typedef void *(*sipInitFunc)(sipWrapper *,PyObject *,sipWrapper **);
+typedef void *(*sipInitFunc)(sipWrapper *,PyObject *,sipWrapper **,int *);
 typedef int (*sipTraverseFunc)(void *, visitproc, void *);
 typedef int (*sipClearFunc)(void *);
+typedef int (*sipBufferFunc)(PyObject *,void *,int,void **);
+typedef int (*sipSegCountFunc)(PyObject *,void *,int *);
 typedef void (*sipDeallocFunc)(sipWrapper *);
 typedef void *(*sipCastFunc)(void *,sipWrapperType *);
 typedef sipWrapperType *(*sipSubClassConvertFunc)(void **);
 typedef void *(*sipForceConvertToFunc)(PyObject *,int *);
-typedef int (*sipConvertToFunc)(PyObject *,void **,int *);
-typedef PyObject *(*sipConvertFromFunc)(void *);
+typedef int (*sipConvertToFunc)(PyObject *,void **,int *,PyObject *);
+typedef PyObject *(*sipConvertFromFunc)(void *,PyObject *);
 typedef int (*sipVirtHandlerFunc)(void *,PyObject *,...);
 typedef int (*sipEmitFunc)(sipWrapper *,PyObject *);
+typedef void (*sipReleaseFunc)(void *, int);
+
+
+/*
+ * The information describing an encoded class ID.
+ */
+typedef struct _sipEncodedClassDef {
+	unsigned sc_class:16;		/* The class number. */
+	unsigned sc_module:8;		/* The module number (255 for this one). */
+	unsigned sc_flag:1;		/* A context specific flag. */
+} sipEncodedClassDef;
 
 
 /*
@@ -181,7 +203,8 @@ typedef struct _sipEnumMemberDef {
 typedef struct _sipEnumDef {
 	const char *e_name;		/* The Python name of the enum. */
 	const char *e_cname;		/* The C/C++ name of the enum, NULL if the same as the Python name. */
-	int e_scope;			/* The nr. of the scoping type. */
+	int e_scope;			/* The scoping type. */
+	struct _sipPySlotDef *e_pyslots;	/* The Python slots. */
 } sipEnumDef;
 
 
@@ -193,20 +216,24 @@ typedef struct _sipInstancesDef {
 	struct _sipVoidPtrInstanceDef *id_voidp;	/* The void *. */
 	struct _sipCharInstanceDef *id_char;	/* The chars. */
 	struct _sipStringInstanceDef *id_string;	/* The strings. */
+	struct _sipIntInstanceDef *id_int;	/* The ints. */
 	struct _sipLongInstanceDef *id_long;	/* The longs. */
+	struct _sipUnsignedLongInstanceDef *id_ulong;	/* The unsigned longs. */
+	struct _sipLongLongInstanceDef *id_llong;	/* The long longs. */
+	struct _sipUnsignedLongLongInstanceDef *id_ullong;	/* The unsigned long longs. */
 	struct _sipDoubleInstanceDef *id_double;	/* The doubles. */
 	struct _sipEnumInstanceDef *id_enum;	/* The enums. */
 } sipInstancesDef;
 
 
 /*
- * The information describing a super-class.
+ * The information describing a type initialiser extender.
  */
-typedef struct _sipSuperClassDef {
-	unsigned sc_class:16;		/* The class number. */
-	unsigned sc_module:8;		/* The module number (255 for this one). */
-	unsigned sc_last:1;		/* The last in the list. */
-} sipSuperClassDef;
+typedef struct _sipInitExtenderDef {
+	sipInitFunc ie_extender;	/* The extender function. */
+	sipEncodedClassDef ie_class;	/* The class being extended. */
+	struct _sipInitExtenderDef *ie_next;	/* The next extender for this class. */
+} sipInitExtenderDef;
 
 
 /*
@@ -214,7 +241,7 @@ typedef struct _sipSuperClassDef {
  */
 typedef struct _sipSubClassConvertorDef {
 	sipSubClassConvertFunc scc_convertor;	/* The convertor. */
-	sipSuperClassDef scc_base;	/* The encoded base type. */
+	sipEncodedClassDef scc_base;	/* The encoded base type. */
 	sipWrapperType *scc_basetype;	/* The base type. */
 } sipSubClassConvertorDef;
 
@@ -225,6 +252,8 @@ typedef struct _sipSubClassConvertorDef {
 typedef enum {
 	str_slot,			/* __str__ */
 	int_slot,			/* __int__ */
+	long_slot,			/* __long__ */
+	float_slot,			/* __float__ */
 	len_slot,			/* __len__ */
 	contains_slot,			/* __contains__ */
 	add_slot,			/* __add__ for number */
@@ -266,6 +295,7 @@ typedef enum {
 	nonzero_slot,			/* __nonzero__ */
 	neg_slot,			/* __neg__ */
 	repr_slot,			/* __repr__ */
+	hash_slot,			/* __hash__ */
 	pos_slot,			/* __pos__ */
 	abs_slot			/* __abs__ */
 } sipPySlotType;
@@ -280,6 +310,16 @@ typedef struct _sipPySlotDef {
 } sipPySlotDef;
 
 
+/*
+ * The information describing a Python slot extender.
+ */
+typedef struct _sipPySlotExtenderDef {
+	void *pse_func;			/* The function. */
+	sipPySlotType pse_type;		/* The type. */
+	sipEncodedClassDef pse_class;	/* The encoded class. */
+} sipPySlotExtenderDef;
+
+
 /*
  * This enumerates the different dynamic signal argument types.
  */
@@ -295,14 +335,22 @@ typedef enum {
 	uint_sat,
 	long_sat,
 	ulong_sat,
+	longlong_sat,
+	ulonglong_sat,
 	float_sat,
 	double_sat,
 	enum_sat,
 	bool_sat,
 	void_sat,
 	class_sat,
+	classp_sat,
 	mtype_sat,
-	qvariant_sat
+	mtypep_sat,
+	qvariant_sat,
+	qvariantp_sat,
+	pyobject_sat,
+	schar_sat,
+	sstring_sat
 } sipSigArgType;
 
 
@@ -325,8 +373,8 @@ typedef struct _sipTypeDef {
 	int td_flags;			/* Type flags, see the sipType*() macros. */
 	const char *td_name;		/* The Python name of the type. */
 	const char *td_cname;		/* The C/C++ name of the type, NULL if the same as the Python name. */
-	int td_scope;			/* The nr. of the scoping type. */
-	sipSuperClassDef *td_supers;	/* The super-types. */
+	sipEncodedClassDef td_scope;	/* The scoping type. */
+	sipEncodedClassDef *td_supers;	/* The super-types. */
 	sipPySlotDef *td_pyslots;	/* The table of Python slots. */
 	int td_nrmethods;		/* The number of lazy methods. */
 	PyMethodDef *td_methods;	/* The table of lazy methods. */
@@ -336,24 +384,51 @@ typedef struct _sipTypeDef {
 	sipInitFunc td_init;		/* The initialisation function. */
 	sipTraverseFunc td_traverse;	/* The traverse function. */
 	sipClearFunc td_clear;		/* The clear function. */
+	sipBufferFunc td_readbuffer;	/* The read buffer function. */
+	sipBufferFunc td_writebuffer;	/* The write buffer function. */
+	sipSegCountFunc td_segcount;	/* The segment count function. */
+	sipBufferFunc td_charbuffer;	/* The char buffer function. */
 	sipDeallocFunc td_dealloc;	/* The deallocation function. */
 	sipCastFunc td_cast;		/* The cast function, 0 if a C struct. */
+	sipReleaseFunc td_release;	/* The release function. */
 	sipForceConvertToFunc td_fcto;	/* The force convert to function, 0 if a C++ namespace. */
 	sipConvertToFunc td_cto;	/* The convert to function. */
 	struct _sipQtSignal *td_emit;	/* Emit table for Qt signals. */
 	sipInstancesDef td_instances;	/* The static instances. */
+	struct _sipTypeDef *td_nsextender;	/* The next namespace extender. */
 } sipTypeDef;
 
 
+/*
+ * The information describing an external type.
+ */
+typedef struct _sipExternalTypeDef {
+	int et_nr;			/* The index into the type table. */
+	const char *et_name;		/* The name of the type. */
+} sipExternalTypeDef;
+
+
 /*
  * The information describing a mapped class.
  */
-typedef struct _sipMappedTypeDef {
+typedef struct _sipMappedType {
 	const char *mt_name;		/* The corresponding C++ definition. */
+	sipReleaseFunc mt_release;	/* The release function. */
 	sipForceConvertToFunc mt_fcto;	/* The force convert to function. */
 	sipConvertToFunc mt_cto;	/* The convert to function. */
 	sipConvertFromFunc mt_cfrom;	/* The convert from function. */
-} sipMappedTypeDef;
+} sipMappedType;
+
+
+/*
+ * Defines an entry in the module specific list of delayed dtor calls.
+ */
+typedef struct _sipDelayedDtor {
+	void *dd_ptr;			/* The C/C++ instance. */
+	const char *dd_name;		/* The class name. */
+	int dd_isderived;		/* Non-zero if dd_ptr is a derived class instance. */
+	struct _sipDelayedDtor *dd_next;	/* Next in the list. */
+} sipDelayedDtor;
 
 
 /*
@@ -373,11 +448,14 @@ typedef struct _sipExportedModuleDef {
 	struct _sipExportedModuleDef *em_next;	/* The next in the list. */
 	unsigned em_api_minor;		/* The SIP API minor version number. */
 	char *em_name;			/* The module name. */
+	PyObject *em_nameobj;		/* The module name as an object. */
 	int em_version;			/* The module version. */
 	sipImportedModuleDef *em_imports;	/* The imported modules. */
-	int em_qobject_class;		/* The index of the QObject class. */
+	struct _sipQtAPI *em_qt_api;	/* The optional Qt support API. */
+	int em_nrtypes;			/* The number of types. */
 	struct _sipWrapperType **em_types;	/* The table of type types. */
-	sipMappedTypeDef **em_mappedtypes;	/* The table of mapped types. */
+	sipExternalTypeDef *em_external;	/* The table of external types. */
+	sipMappedType **em_mappedtypes;	/* The table of mapped types. */
 	int em_nrenums;			/* The number of enums. */
 	PyTypeObject **em_enums;	/* The table of enum types. */
 	sipEnumDef *em_enumdefs;	/* The table of enum type data. */
@@ -389,6 +467,10 @@ typedef struct _sipExportedModuleDef {
 	sipInstancesDef em_instances;	/* The static instances. */
 	struct _sipLicenseDef *em_license;	/* The license. */
 	PyObject **em_exceptions;	/* The table of exception types. */
+	sipPySlotExtenderDef *em_slotextend;	/* The table of Python slot extenders. */
+	sipInitExtenderDef *em_initextend;	/* The table of initialiser extenders. */
+	void (*em_delayeddtors)(const sipDelayedDtor *);	/* The delayed dtor handler. */
+	sipDelayedDtor *em_ddlist;	/* The list of delayed dtors. */
 } sipExportedModuleDef;
 
 
@@ -431,6 +513,15 @@ typedef struct _sipStringInstanceDef {
 } sipStringInstanceDef;
 
 
+/*
+ * The information describing an int instance to be added to a dictionary.
+ */
+typedef struct _sipIntInstanceDef {
+	char *ii_name;			/* The int name. */
+	int ii_val;			/* The int value. */
+} sipIntInstanceDef;
+
+
 /*
  * The information describing a long instance to be added to a dictionary.
  */
@@ -440,6 +531,43 @@ typedef struct _sipLongInstanceDef {
 } sipLongInstanceDef;
 
 
+/*
+ * The information describing an unsigned long instance to be added to a
+ * dictionary.
+ */
+typedef struct _sipUnsignedLongInstanceDef {
+	char *uli_name;			/* The unsigned long name. */
+	unsigned long uli_val;		/* The unsigned long value. */
+} sipUnsignedLongInstanceDef;
+
+
+/*
+ * The information describing a long long instance to be added to a dictionary.
+ */
+typedef struct _sipLongLongInstanceDef {
+	char *lli_name;			/* The long long name. */
+#if defined(HAVE_LONG_LONG)
+	PY_LONG_LONG lli_val;		/* The long long value. */
+#else
+	long lli_val;
+#endif
+} sipLongLongInstanceDef;
+
+
+/*
+ * The information describing an unsigned long long instance to be added to a
+ * dictionary.
+ */
+typedef struct _sipUnsignedLongLongInstanceDef {
+	char *ulli_name;		/* The unsigned long long name. */
+#if defined(HAVE_LONG_LONG)
+	unsigned PY_LONG_LONG ulli_val;	/* The unsigned long long value. */
+#else
+	unsigned long ulli_val;
+#endif
+} sipUnsignedLongLongInstanceDef;
+
+
 /*
  * The information describing a double instance to be added to a dictionary.
  */
@@ -540,6 +668,41 @@ typedef struct _sipPySig {
 } sipPySig;
 
 
+/*
+ * This defines a single dynamic signal argument type.
+ */
+typedef struct _sipSigArg {
+	sipSigArgType atype;		/* The type. */
+
+	union {
+		sipWrapperType *wt;	/* The Python type for classes. */
+		sipMappedType *mt;	/* The data for mapped types. */
+		PyTypeObject *et;	/* The Python type for named enums. */
+	} u;
+} sipSigArg;
+
+
+/*
+ * A parsed signal signature.
+ */
+typedef struct _sipSignature {
+	int sg_nrargs;			/* The number of arguments. */
+	sipSigArg *sg_args;		/* The parsed arguments (heap). */
+	char *sg_signature;		/* The unparsed signature (heap). */
+	struct _sipSignature *sg_next;	/* The next in the list. */
+} sipSignature;
+
+
+/*
+ * A connection to a universal slot.
+ */
+typedef struct _sipSlotConnection {
+	void *sc_transmitter;		/* The transmitter QObject. */
+	const sipSignature *sc_signature;	/* The parsed signature. */
+	sipSlot sc_slot;		/* The slot. */
+} sipSlotConnection;
+
+
 /*
  * Maps the name of a Qt signal to a wrapper function to emit it.
  */
@@ -554,52 +717,77 @@ typedef struct _sipQtSignal {
  * functions that can be used by generated code.
  */
 typedef struct _sipAPIDef {
+	/*
+	 * This must be the first entry and it's signature must not change so
+	 * that version number mismatches can be detected and reported.
+	 */
+	int (*api_export_module)(sipExportedModuleDef *client,unsigned api_major,unsigned api_minor,PyObject *mod_dict);
 	/*
 	 * The following are part of the public API.
 	 */
 	void (*api_bad_catcher_result)(PyObject *method);
 	void (*api_bad_length_for_slice)(int seqlen,int slicelen);
-	PyObject *(*api_build_result)(int *isErr,char *fmt,...);
-	PyObject *(*api_call_method)(int *isErr,PyObject *method,char *fmt,...);
+	PyObject *(*api_build_result)(int *isErr, const char *fmt, ...);
+	PyObject *(*api_call_method)(int *isErr, PyObject *method, const char *fmt, ...);
 	PyObject *(*api_class_name)(PyObject *self);
-	PyObject *(*api_connect_rx)(PyObject *txObj,const char *sig,PyObject *rxObj,const char *slot);
+	PyObject *(*api_connect_rx)(PyObject *txObj,const char *sig,PyObject *rxObj,const char *slot, int type);
 	int (*api_convert_from_sequence_index)(int idx,int len);
+	int (*api_can_convert_to_instance)(PyObject *pyObj, sipWrapperType *type, int flags);
+	int (*api_can_convert_to_mapped_type)(PyObject *pyObj, const sipMappedType *mt, int flags);
+	void *(*api_convert_to_instance)(PyObject *pyObj, sipWrapperType *type, PyObject *transferObj, int flags, int *statep, int *iserrp);
+	void *(*api_convert_to_mapped_type)(PyObject *pyObj, const sipMappedType *mt, PyObject *transferObj, int flags, int *statep, int *iserrp);
+	void *(*api_force_convert_to_instance)(PyObject *pyObj, sipWrapperType *type, PyObject *transferObj, int flags, int *statep, int *iserrp);
+	void *(*api_force_convert_to_mapped_type)(PyObject *pyObj, const sipMappedType *mt, PyObject *transferObj, int flags, int *statep, int *iserrp);
+	void (*api_release_instance)(void *cpp, sipWrapperType *type, int state);
+	void (*api_release_mapped_type)(void *cpp, const sipMappedType *mt, int state);
+	PyObject *(*api_convert_from_instance)(void *cpp, sipWrapperType *type, PyObject *transferObj);
+	PyObject *(*api_convert_from_new_instance)(void *cpp, sipWrapperType *type, PyObject *transferObj);
+	PyObject *(*api_convert_from_mapped_type)(void *cpp, const sipMappedType *mt, PyObject *transferObj);
 	void *(*api_convert_to_cpp)(PyObject *sipSelf,sipWrapperType *type,int *iserrp);
+	int (*api_get_state)(PyObject *transferObj);
+	const sipMappedType *(*api_find_mapped_type)(const char *type);
 	PyObject *(*api_disconnect_rx)(PyObject *txObj,const char *sig,PyObject *rxObj,const char *slot);
 	int (*api_emit_signal)(PyObject *self,const char *sig,PyObject *sigargs);
 	void (*api_free)(void *mem);
-	void *(*api_get_sender)(void);
+	PyObject *(*api_get_sender)();
 	PyObject *(*api_get_wrapper)(void *cppPtr,sipWrapperType *type);
 	void *(*api_malloc)(size_t nbytes);
 	sipWrapperType *(*api_map_int_to_class)(int typeInt,const sipIntTypeClassMap *map,int maplen);
 	sipWrapperType *(*api_map_string_to_class)(const char *typeString,const sipStringTypeClassMap *map,int maplen);
-	int (*api_parse_result)(int *isErr,PyObject *method,PyObject *res,char *fmt,...);
+	int (*api_parse_result)(int *isErr, PyObject *method, PyObject *res, const char *fmt, ...);
 	void (*api_trace)(unsigned mask,const char *fmt,...);
 	void (*api_transfer)(PyObject *self,int toCpp);
 	void (*api_transfer_back)(PyObject *self);
 	void (*api_transfer_to)(PyObject *self,PyObject *owner);
 	int (*api_wrapper_check)(PyObject *o);
+	unsigned long (*api_long_as_unsigned_long)(PyObject *o);
+	PyObject *(*api_convert_from_named_enum)(int eval, PyTypeObject *et);
+	/*
+	 * The following may be used by Qt support code but no other
+	 * handwritten code.
+	 */
+	PyObject *(*api_convert_from_void_ptr)(void *val);
+	void (*api_free_connection)(sipSlotConnection *conn);
+	int (*api_emit_to_slot)(sipSlot *slot, PyObject *sigargs);
+	int (*api_same_connection)(sipSlotConnection *conn, void *tx, const char *sig, PyObject *rxObj, const char *slot);
+	void *(*api_convert_rx)(sipWrapper *txSelf, const char *sigargs, PyObject *rxObj, const char *slot, const char **memberp);
 	/*
 	 * The following are not part of the public API.
 	 */
-	int (*api_export_module)(sipExportedModuleDef *client,unsigned api_major,unsigned api_minor,PyObject *mod_dict);
-	int (*api_parse_args)(int *argsParsedp,PyObject *sipArgs,char *fmt,...);
+	int (*api_parse_args)(int *argsParsedp, PyObject *sipArgs,
+			const char *fmt, ...);
+	int (*api_parse_pair)(int *argsParsedp,PyObject *arg0,PyObject *arg1,char *fmt,...);
 	void (*api_common_ctor)(sipMethodCache *cache,int nrmeths);
 	void (*api_common_dtor)(sipWrapper *sipSelf);
-	PyObject *(*api_convert_from_void_ptr)(void *val);
 	void *(*api_convert_to_void_ptr)(PyObject *obj);
-	void (*api_no_ctor)(int argsParsed,char *classname);
-	void (*api_no_function)(int argsParsed,char *func);
-	void (*api_no_method)(int argsParsed,char *classname,char *method);
+	void (*api_no_function)(int argsParsed, const char *func);
+	void (*api_no_method)(int argsParsed, const char *classname, const char *method);
+	void (*api_abstract_method)(const char *classname, const char *method);
 	void (*api_bad_class)(const char *classname);
 	void (*api_bad_set_type)(const char *classname,const char *var);
 	void *(*api_get_cpp_ptr)(sipWrapper *w,sipWrapperType *type);
 	void *(*api_get_complex_cpp_ptr)(sipWrapper *w);
 	PyObject *(*api_is_py_method)(sip_gilstate_t *gil,sipMethodCache *pymc,sipWrapper *sipSelf,char *cname,char *mname);
-	PyObject *(*api_map_cpp_to_self)(void *cppPtr,sipWrapperType *type);
-	PyObject *(*api_map_cpp_to_self_sub_class)(void *cppPtr,sipWrapperType *type);
-	PyObject *(*api_new_cpp_to_self)(void *cppPtr,sipWrapperType *type,sipWrapper *owner);
-	PyObject *(*api_new_cpp_to_self_sub_class)(void *cppPtr,sipWrapperType *type,sipWrapper *owner);
 	void (*api_call_hook)(char *hookname);
 	void (*api_start_thread)(void);
 	void (*api_end_thread)(void);
@@ -607,33 +795,83 @@ typedef struct _sipAPIDef {
 	void (*api_raise_class_exception)(sipWrapperType *type,void *ptr);
 	void (*api_raise_sub_class_exception)(sipWrapperType *type,void *ptr);
 	int (*api_add_class_instance)(PyObject *dict,char *name,void *cppPtr,sipWrapperType *wt);
-	PyObject *(*api_convert_from_named_enum)(int eval, PyTypeObject *et);
+	int (*api_add_enum_instance)(PyObject *dict,char *name,int value,PyTypeObject *type);
 	void (*api_bad_operator_arg)(PyObject *self, PyObject *arg, sipPySlotType st);
+	PyObject *(*api_pyslot_extend)(sipExportedModuleDef *mod, sipPySlotType st, sipWrapperType *type, PyObject *arg0, PyObject *arg1);
+	void (*api_add_delayed_dtor)(sipWrapper *w);
+	int (*api_add_mapped_type_instance)(PyObject *dict, char *name,
+			void *cppPtr, const sipMappedType *mt);
+	/*
+	 * The following are part of the public API.
+	 */
+	int (*api_export_symbol)(const char *name, void *sym);
+	void *(*api_import_symbol)(const char *name);
 } sipAPIDef;
 
 
+/*
+ * The API implementing the optional Qt support.
+ */
+typedef struct _sipQtAPI {
+	struct _sipWrapperType **qt_qobject;	/* The QObject type. */
+	int (*qt_is_qt_signal)(void *, const char *);
+	void *(*qt_create_universal_signal_shortcut)(void *, const char *, const char **);
+	void *(*qt_create_universal_signal)(void *, const struct _sipSignature *);
+	void *(*qt_find_universal_signal_shortcut)(void *, const char *, const char **);
+	void *(*qt_find_universal_signal)(void *, const struct _sipSignature *);
+	int (*qt_emit_signal_shortcut)(void *, const char *, PyObject *);
+	int (*qt_emit_signal)(void *, const struct _sipSignature *, PyObject *);
+	void *(*qt_create_universal_slot)(struct _sipWrapper *, struct _sipSlotConnection *, const char **);
+	void (*qt_destroy_universal_slot)(void *);
+	void *(*qt_find_slot)(void *, const char *, PyObject *, const char *, const char **);
+	int (*qt_connect)(void *, const char *, void *, const char *, int);
+	int (*qt_disconnect)(void *, const char *, void *, const char *);
+	int (*qt_signals_blocked)(void *);
+	const void *(*qt_get_sender)();
+	void (*qt_forget_sender)();
+	int (*qt_same_name)(const char *, const char *);
+} sipQtAPI;
+
+
+/*
+ * These are flags that can be passed to sipCanConvertToInstance(),
+ * sipConvertToInstance() and sipForceConvertToInstance().
+ */
+#define	SIP_NOT_NONE		0x01	/* Disallow None. */
+#define	SIP_NO_CONVERTORS	0x02	/* Disable any type convertors. */
+
+
+/*
+ * These are the state flags returned by %ConvertToTypeCode.  Note that these
+ * share the same "namespace" as the flags below.
+ */
+#define	SIP_TEMPORARY		0x01	/* A temporary instance. */
+#define	SIP_DERIVED_CLASS	0x02	/* The instance is derived. */
+
+
 /*
  * Useful macros, not part of the public API.
  */
-#define	SIP_PY_OWNED	0x01		/* Owned by Python. */
-#define	SIP_SIMPLE	0x02		/* If the instance is simple. */
-#define	SIP_INDIRECT	0x04		/* If there is a level of indirection. */
-#define	SIP_ACCFUNC	0x08		/* If there is an access function. */
-#define	SIP_NOT_IN_MAP	0x20		/* If Python object not in the map. */
-#define	SIP_SHARE_MAP	0x40		/* If the map slot might be occupied. */
+#define	SIP_PY_OWNED		0x04	/* Owned by Python. */
+#define	SIP_INDIRECT		0x08	/* If there is a level of indirection. */
+#define	SIP_ACCFUNC		0x10	/* If there is an access function. */
+#define	SIP_NOT_IN_MAP		0x20	/* If Python object not in the map. */
+#define	SIP_SHARE_MAP		0x40	/* If the map slot might be occupied. */
 
 #define	sipIsPyOwned(w)		((w) -> flags & SIP_PY_OWNED)
 #define	sipSetPyOwned(w)	((w) -> flags |= SIP_PY_OWNED)
 #define	sipResetPyOwned(w)	((w) -> flags &= ~SIP_PY_OWNED)
-#define	sipIsSimple(w)		((w) -> flags & SIP_SIMPLE)
+#define	sipIsDerived(w)		((w) -> flags & SIP_DERIVED_CLASS)
 #define	sipIsIndirect(w)	((w) -> flags & SIP_INDIRECT)
 #define	sipIsAccessFunc(w)	((w) -> flags & SIP_ACCFUNC)
 #define	sipNotInMap(w)		((w) -> flags & SIP_NOT_IN_MAP)
 
 
 #define	SIP_TYPE_ABSTRACT	0x01	/* If the type is abstract. */
+#define	SIP_TYPE_SCC		0x02	/* If the type is subject to sub-class convertors. */
 
 #define	sipTypeIsAbstract(wt)	((wt)->type->td_flags & SIP_TYPE_ABSTRACT)
+#define	sipTypeHasSCC(wt)	((wt)->type->td_flags & SIP_TYPE_SCC)
 
 
 #ifdef __cplusplus
